@@ -5,6 +5,7 @@ For interactions with the NiFi Registry Service and related functions
 """
 
 from __future__ import absolute_import
+import logging
 import six
 import nipyapi
 # Due to line lengths, creating shortened names for these objects
@@ -22,6 +23,8 @@ __all__ = [
     'deploy_flow_version'
 ]
 
+log = logging.getLogger(__name__)
+
 
 def create_registry_client(name, uri, description):
     """
@@ -35,6 +38,9 @@ def create_registry_client(name, uri, description):
     Returns:
         (RegistryClientEntity): The new registry client object
     """
+    assert isinstance(uri, six.string_types) and uri is not False
+    assert isinstance(name, six.string_types) and name is not False
+    assert isinstance(description, six.string_types)
     try:
         return nipyapi.nifi.ControllerApi().create_registry_client(
             body={
@@ -52,20 +58,28 @@ def create_registry_client(name, uri, description):
         raise ValueError(e.body)
 
 
-def delete_registry_client(client):
+def delete_registry_client(client, refresh=True):
     """
     Deletes a Registry Client from the list of NiFI Controller Services
 
     Args:
         client (RegistryClientEntity): The client to delete
+        refresh (bool): Whether to refresh the object before action
 
     Returns:
         (RegistryClientEntity): The updated client object
     """
+    assert isinstance(client, nipyapi.nifi.RegistryClientEntity)
     try:
+        if refresh:
+            target = nipyapi.nifi.ControllerApi().get_registry_client(
+                client.id
+            )
+        else:
+            target = client
         return nipyapi.nifi.ControllerApi().delete_registry_client(
-            id=client.id,
-            version=client.revision.version
+            id=target.id,
+            version=target.revision.version
         )
     except (nipyapi.nifi.rest.ApiException, AttributeError) as e:
         raise ValueError(e)
@@ -127,11 +141,15 @@ def create_registry_bucket(name):
         (Bucket): The new Bucket object
     """
     try:
-        return nipyapi.registry.BucketsApi().create_bucket(
+        bucket = nipyapi.registry.BucketsApi().create_bucket(
             body={
                 'name': name
             }
         )
+        log.debug("Created bucket %s against registry connection at %s",
+                  bucket.identifier,
+                  nipyapi.config.registry_config.api_client.host)
+        return bucket
     except nipyapi.registry.rest.ApiException as e:
         raise ValueError(e.body)
 
@@ -221,9 +239,9 @@ def save_flow_ver(process_group, registry_client, bucket, flow_name=None,
         registry_client (RegistryClient): The Client linked to the Registry
             which contains the Bucket to save to
         bucket (Bucket): the Bucket on the NiFi Registry to save to
+        flow_name (str): A name for the VersionedFlow in the Bucket
             Note you need either a name for a new VersionedFlow, or the ID of
             an existing one to save a new version
-        flow_name (str): A name for the VersionedFlow in the Bucket
         flow_id (Optional [str]): Identifier of an existing VersionedFlow in
             the bucket, if saving a new version to an existing flow
         comment (str): A comment for the version commit
@@ -305,7 +323,8 @@ def revert_flow_ver(process_group):
         raise ValueError(e)
 
 
-def list_flow_versions(bucket_id, flow_id):
+def list_flow_versions(bucket_id, flow_id, registry_id=None,
+                       service='registry'):
     """
     EXPERIMENTAL
     List all the versions of a given Flow in a given Bucket
@@ -313,17 +332,33 @@ def list_flow_versions(bucket_id, flow_id):
     Args:
         bucket_id (str): UUID of the bucket holding the flow to be enumerated
         flow_id (str): UUID of the flow in the bucket to be enumerated
+        registry_id (str): UUID of the registry client linking the bucket, only
+            required if requesting flows via NiFi instead of directly Registry
+        service (str): Accepts 'nifi' or 'registry', indicating which service
+            to query
 
     Returns:
-        list(VersionedFlowSnapshotMetadata)
+        list(VersionedFlowSnapshotMetadata) or
+            (VersionedFlowSnapshotMetadataSetEntity)
     """
-    try:
-        return nipyapi.registry.BucketFlowsApi().get_flow_versions(
-            bucket_id=bucket_id,
-            flow_id=flow_id
-        )
-    except nipyapi.registry.rest.ApiException as e:
-        raise ValueError(e.body)
+    assert service in ['nifi', 'registry']
+    if service == 'nifi':
+        try:
+            return nipyapi.nifi.FlowApi().get_versions(
+                registry_id=registry_id,
+                bucket_id=bucket_id,
+                flow_id=flow_id
+            )
+        except nipyapi.nifi.rest.ApiException as e:
+            raise ValueError(e.body)
+    else:
+        try:
+            return nipyapi.registry.BucketFlowsApi().get_flow_versions(
+                bucket_id=bucket_id,
+                flow_id=flow_id
+            )
+        except nipyapi.registry.rest.ApiException as e:
+            raise ValueError(e.body)
 
 
 def update_flow_ver(process_group, target_version=None):
@@ -351,14 +386,12 @@ def update_flow_ver(process_group, target_version=None):
         )
         if not status.request.complete:
             return False
-        else:
-            if status.request.failure_reason is None:
-                return True
-            else:
-                raise ValueError(
-                    "Flow Version Update did not complete successfully. "
-                    "Error text {0}".format(status.request.failure_reason)
-                )
+        if status.request.failure_reason is None:
+            return True
+        raise ValueError(
+            "Flow Version Update did not complete successfully. "
+            "Error text {0}".format(status.request.failure_reason)
+        )
     try:
         vci = get_version_info(process_group)
         assert isinstance(vci, nipyapi.nifi.VersionControlInformationEntity)
@@ -476,13 +509,10 @@ def create_flow_version(flow, flow_snapshot, refresh=True):
     Flow Version. Useful in migrating Flow Versions between environments.
 
     Args:
-        bucket_id (str): Deprecated, now pulled from the flow parameter
         flow (VersionedFlowObject): the VersionedFlow object to write to
         flow_snapshot (VersionedFlowSnapshot): the Snapshot to write into the
             VersionedFlow
         refresh (bool): Whether to refresh the object status before actioning
-        raw_snapshot (bool): Deprecated, as not using a full snapshot resulted
-            in inconsistent behavior
 
     Returns:
         The new (VersionedFlowSnapshot)
@@ -544,18 +574,22 @@ def get_flow_version(bucket_id, flow_id, version=None, export=False):
 
     WARNING: This call is impacted by
     https://issues.apache.org/jira/browse/NIFIREG-135
-    Which means you can't trust the version count
+    Which means you sometimes can't trust the version count
     """
     assert isinstance(bucket_id, six.string_types)
     assert isinstance(flow_id, six.string_types)
-    assert version is None or isinstance(version, six.string_types)
+    # Version needs to be coerced to str pass API client regex test
+    # Even though the client specifies it as Int
+    assert version is None or isinstance(
+        version, (six.string_types, six.integer_types)
+    )
     assert isinstance(export, bool)
     if version:
         try:
             out = nipyapi.registry.BucketFlowsApi().get_flow_version(
                 bucket_id=bucket_id,
                 flow_id=flow_id,
-                version_number=version,
+                version_number=str(version),  # This str coercion is intended
                 _preload_content=not export
             )
         except nipyapi.registry.rest.ApiException as e:
@@ -698,27 +732,48 @@ def deploy_flow_version(parent_id, location, bucket_id, flow_id, reg_client_id,
             new process group in.
         location (tuple[x, y]): the x,y coordinates to place the new Process
             Group under the parent
-        bucket_id (str): ID of the bucket containing the versioned flow to deploy.
+        bucket_id (str): ID of the bucket containing the versioned flow to
+            deploy.
+        reg_client_id (str): ID of the registry client connection to use.
         flow_id (str): ID of the versioned flow to deploy.
-        registry_id (str): ID of the Registry client configured in NiFi.
-        version (Optional [int]): version to deploy, if not provided latest version
-            will be deployed.
+        version (Optional [int,str]): version to deploy, if not provided latest
+            version will be deployed.
 
     Returns:
         (ProcessGroupEntity) of the newly deployed Process Group
     """
     assert isinstance(location, tuple)
-    try:
-        # Being pedantic about checking this as API failure errors are terse
-        # Check flow details are valid
-        target_flow = get_flow_version(
-            bucket_id=bucket_id,
-            flow_id=flow_id,
-            version=version
+    # check reg client is valid
+    target_reg_client = get_registry_client(reg_client_id, 'id')
+    # Being pedantic about checking this as API failure errors are terse
+    # Using NiFi here to keep all calls within the same API client
+    flow_versions = list_flow_versions(
+        bucket_id=bucket_id,
+        flow_id=flow_id,
+        registry_id=reg_client_id,
+        service='nifi'
+    )
+    if not flow_versions:
+        raise ValueError("Could not find Flows matching Bucket ID [{0}] and"
+                         "Flow ID [{1}] on Registry Client [{2}]"
+                         .format(bucket_id, flow_id, reg_client_id))
+    if version is None:
+        target_flow = flow_versions.versioned_flow_snapshot_metadata_set
+    else:
+        target_flow = [x for x
+                       in flow_versions.versioned_flow_snapshot_metadata_set
+                       if x.versioned_flow_snapshot_metadata.version == version
+                       ]
+    if not target_flow:
+        raise ValueError(
+            "Could not find Version [{0}] for Flow [{1}] in Bucket [{2}] on "
+            "Registry Client [{3}]"
+            .format(str(version), flow_id, bucket_id, reg_client_id)
         )
-        # check reg client is valid
-        target_reg_client = get_registry_client(reg_client_id, 'id')
-        # Issue deploy statement
+    else:
+        target_flow = target_flow[0].versioned_flow_snapshot_metadata
+    # Issue deploy statement
+    try:
         return nipyapi.nifi.ProcessGroupsApi().create_process_group(
             id=parent_id,
             body=nipyapi.nifi.ProcessGroupEntity(
@@ -731,10 +786,10 @@ def deploy_flow_version(parent_id, location, bucket_id, flow_id, reg_client_id,
                         y=float(location[1])
                     ),
                     version_control_information=VciDTO(
-                        bucket_id=target_flow.bucket.identifier,
-                        flow_id=target_flow.flow.identifier,
+                        bucket_id=target_flow.bucket_identifier,
+                        flow_id=target_flow.flow_identifier,
                         registry_id=target_reg_client.id,
-                        version=target_flow.snapshot_metadata.version
+                        version=target_flow.version
                     )
                 )
             )
